//  Copyright (c) 2021 NetEase, Inc.  All rights reserved.

#import "NEScreenShareHost.h"
#import <UIKit/UIKit.h>
#import <ReplayKit/ReplayKit.h>
#import "NEScreenShareHostPrivateDefine.h"
#import "NEScreenShareHostProtocol.h"
#import "BroadcastExtensionLauncher.h"

static void *KVOContext = &KVOContext;

static void onBroadcastStarted(CFNotificationCenterRef center,
                               void *observer,
                               CFStringRef name,
                               const void *object,
                               CFDictionaryRef
                               userInfo) {
    NEScreenShareHost *host = (__bridge NEScreenShareHost *)(observer);
    id<NEScreenShareHostDelegate> delegate = [[host valueForKey:@"options"] delegate];
    if (delegate && [delegate respondsToSelector:@selector(onBroadcastStarted)]) {
        [delegate onBroadcastStarted];
    }
}

static void onBroadcastFinished(CFNotificationCenterRef center,
                                void *observer,
                                CFStringRef name,
                                const void *object,
                                CFDictionaryRef
                                userInfo) {
    NEScreenShareHost *host = (__bridge NEScreenShareHost *)(observer);
    id<NEScreenShareHostDelegate> delegate = [[host valueForKey:@"options"] delegate];
    if (delegate && [delegate respondsToSelector:@selector(onBroadcastFinished)]) {
        [delegate onBroadcastFinished];
    }
}

static void onBroadcastPaused(CFNotificationCenterRef center,
                              void *observer,
                              CFStringRef name,
                              const void *object,
                              CFDictionaryRef
                              userInfo) {
    NEScreenShareHost *host = (__bridge NEScreenShareHost *)(observer);
    id<NEScreenShareHostDelegate> delegate = [[host valueForKey:@"options"] delegate];
    if (delegate && [delegate respondsToSelector:@selector(onBroadcastPaused)]) {
        [delegate onBroadcastPaused];
    }
}

static void onBroadcastResumed(CFNotificationCenterRef center,
                               void *observer,
                               CFStringRef name,
                               const void *object,
                               CFDictionaryRef
                               userInfo) {
    NEScreenShareHost *host = (__bridge NEScreenShareHost *)(observer);
    id<NEScreenShareHostDelegate> delegate = [[host valueForKey:@"options"] delegate];
    if (delegate && [delegate respondsToSelector:@selector(onBroadcastResumed)]) {
        [delegate onBroadcastResumed];
    }
}


@interface NEScreenShareHost () <NEScreenShareHostProtocol>

@property (nonatomic, strong) NEScreenShareHostOptions *options;
@property (nonatomic, strong) NSUserDefaults *userDefaults;
@property (nonatomic, strong) RPSystemBroadcastPickerView *systemPicker API_AVAILABLE(ios(12.0));

@end

@implementation NEScreenShareHost

- (instancetype)initWithOptions:(nonnull NEScreenShareHostOptions *)options {
    self = [super init];
    if (self) {
        NSAssert(options.appGroup.length, @"请输入合理的appGroup");
        NSAssert(options.delegate, @"请添加delegate");
        self.options = options;
        self.userDefaults = [[NSUserDefaults alloc] initWithSuiteName:options.appGroup];
        [self.userDefaults addObserver:self forKeyPath:NEScreenShareVideoFrameKey options:NSKeyValueObservingOptionNew context:KVOContext];
        CFNotificationCenterAddObserver(CFNotificationCenterGetDarwinNotifyCenter(),
                                        (__bridge const void *)(self),
                                        onBroadcastStarted,
                                        (__bridge CFStringRef)NEScreenShareBroadcastStartedNotification,
                                        NULL,
                                        CFNotificationSuspensionBehaviorDeliverImmediately);
        CFNotificationCenterAddObserver(CFNotificationCenterGetDarwinNotifyCenter(),
                                        (__bridge const void *)(self),
                                        onBroadcastFinished,
                                        (__bridge CFStringRef)NEScreenShareBroadcastFinishedNotification,
                                        NULL,
                                        CFNotificationSuspensionBehaviorDeliverImmediately);
        CFNotificationCenterAddObserver(CFNotificationCenterGetDarwinNotifyCenter(),
                                        (__bridge const void *)(self),
                                        onBroadcastPaused,
                                        (__bridge CFStringRef)NEScreenShareBroadcastPausedNotification,
                                        NULL,
                                        CFNotificationSuspensionBehaviorDeliverImmediately);
        CFNotificationCenterAddObserver(CFNotificationCenterGetDarwinNotifyCenter(),
                                        (__bridge const void *)(self),
                                        onBroadcastResumed,
                                        (__bridge CFStringRef)NEScreenShareBroadcastResumedNotification,
                                        NULL,
                                        CFNotificationSuspensionBehaviorDeliverImmediately);
    }
    return self;
}

- (void)dealloc {
    [self.userDefaults removeObserver:self forKeyPath:NEScreenShareVideoFrameKey];
    CFNotificationCenterRemoveObserver(CFNotificationCenterGetDarwinNotifyCenter(), (__bridge const void *)(self), (__bridge CFStringRef)NEScreenShareBroadcastStartedNotification, NULL);
    CFNotificationCenterRemoveObserver(CFNotificationCenterGetDarwinNotifyCenter(), (__bridge const void *)(self), (__bridge CFStringRef)NEScreenShareBroadcastFinishedNotification, NULL);
    CFNotificationCenterRemoveObserver(CFNotificationCenterGetDarwinNotifyCenter(), (__bridge const void *)(self), (__bridge CFStringRef)NEScreenShareBroadcastPausedNotification, NULL);
    CFNotificationCenterRemoveObserver(CFNotificationCenterGetDarwinNotifyCenter(), (__bridge const void *)(self), (__bridge CFStringRef)NEScreenShareBroadcastResumedNotification, NULL);
    
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary<NSKeyValueChangeKey,id> *)change context:(void *)context {
    if (context != KVOContext) {
        return [super observeValueForKeyPath:keyPath ofObject:object change:change context:context];
    }
    if ([keyPath isEqualToString:NEScreenShareVideoFrameKey] && self.options.delegate) {
        NSDictionary *i420Frame = change[NSKeyValueChangeNewKey];
        [self.options.delegate onReceiveVideoFrameInfo:i420Frame];
    }
}

- (void)launchBroadcaster API_AVAILABLE(ios(12.0)) {
    [BroadcastExtensionLauncher launch];
//    NSString *plugInPath = NSBundle.mainBundle.builtInPlugInsPath;
//    if (!plugInPath) {
//        return;
//    }
//    NSArray *contents = [NSFileManager.defaultManager contentsOfDirectoryAtPath:plugInPath error:nil];
//    for (NSString *content in contents) {
//        NSURL *url = [NSURL fileURLWithPath:plugInPath];
//        NSBundle *bundle = [NSBundle bundleWithPath:[url URLByAppendingPathComponent:content].path];
//
//        NSDictionary *extension = [bundle.infoDictionary objectForKey:@"NSExtension"];
//        if (extension == nil) { continue; }
//        NSString *identifier = [extension objectForKey:@"NSExtensionPointIdentifier"];
//        if ([identifier isEqualToString:@"com.apple.broadcast-services-upload"]) {
//            self.systemPicker.preferredExtension = bundle.bundleIdentifier;
//            break;
//        }
//    }
//    for (UIView *view in self.systemPicker.subviews) {
//        UIButton *button = (UIButton *)view;
//        [button sendActionsForControlEvents:UIControlEventAllTouchEvents];
//    }
}

- (void)stopBroadcaster {
    CFNotificationName notificationName = CFNotificationName(NEScreenShareHostRequestStopNotification);
    CFNotificationCenterPostNotification(CFNotificationCenterGetDarwinNotifyCenter(), notificationName, nil, nil, true);
}

//- (RPSystemBroadcastPickerView *)systemPicker {
//    if (!_systemPicker) {
//        RPSystemBroadcastPickerView* picker =
//        [[RPSystemBroadcastPickerView alloc] initWithFrame:CGRectMake(0, 0, 44, 44)];
//        picker.showsMicrophoneButton = NO;
//        picker.autoresizingMask = UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleRightMargin;
//        _systemPicker = picker;
//    }
//    return _systemPicker;
//}

@end
